/*
 * mvs-minfo-provider-service.c
 *
 * This file is part of maevies
 * Copyright (C) 2010 Simón Pena <spenap@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 */

#include <dbus/dbus-glib-bindings.h>

#include "mvs-minfo-provider-service.h"
#include "mvs-minfo-provider.h"
#include "mvs-tmdb-movie-service.h"
#include "mvs-tmdb-movie.h"

#define MINFO_PROVIDER_SERVICE_OBJECT_PATH "/MInfoProvider"
#define MINFO_PROVIDER_SERVICE_NAME "com.simonpena.maevies.minfoprovider"

G_DEFINE_TYPE (MvsMInfoProviderService, mvs_minfo_provider_service, G_TYPE_OBJECT)

enum {
        PROP_0,
        PROP_DBUSGCONN,
};

enum {
        RESPONSE_RECEIVED,
        LAST_SIGNAL
};

static guint
mvs_minfo_provider_service_signals[LAST_SIGNAL] = { 0 };

#define GET_PRIVATE(o) \
        (G_TYPE_INSTANCE_GET_PRIVATE ((o), MVS_TYPE_MINFO_PROVIDER_SERVICE, MvsMInfoProviderServicePrivate))

struct _MvsMInfoProviderServicePrivate {
        MvsMInfoProvider *minfo_provider;
        DBusGConnection *connection;
        guint search_id;
};

gboolean
mvs_minfo_provider_service_query (MvsMInfoProviderService *self,
                                  MvsService service,
                                  const gchar *query,
                                  GError **error)
{
        return mvs_minfo_provider_query (self->priv->minfo_provider,
                        service, query);
}

#include "mvs-minfo-provider-service-glue.h"

static void
response_received_cb (MvsMInfoProvider *provider, GList *list,
                      gpointer user_data)
{
        MvsMInfoProviderService *self = MVS_MINFO_PROVIDER_SERVICE (user_data);
        MvsTmdbMovieService *movie = NULL;
        GError *error = NULL;
        GList *iter = NULL;
        gchar  **object_paths= g_new0 (gchar*, g_list_length (list) + 1);
        guint i = 0;

        for (iter = list; iter; iter = iter->next) {
                if (MVS_IS_TMDB_MOVIE (iter->data)) {
                        MvsTmdbMovie *tmdb_movie = MVS_TMDB_MOVIE (iter->data);
                        gchar *uid_suffix = g_strdup_printf ("%d_%s",
                                        self->priv->search_id,
                                        mvs_tmdb_movie_get_id (tmdb_movie));

                        movie = mvs_tmdb_movie_service_new (self->priv->connection,
                                        tmdb_movie, uid_suffix);
                        object_paths[i] = g_strdup_printf ("/TMDBMovie/%s",
                                        uid_suffix);
                        g_free (uid_suffix);
                }
                i++;
        }
        object_paths[i] = NULL;

        g_signal_emit (self, mvs_minfo_provider_service_signals[RESPONSE_RECEIVED], 0, object_paths);
        g_strfreev (object_paths);
        self->priv->search_id++;
}

static void
setup_dbus (MvsMInfoProviderService *self)
{
        DBusGProxy *proxy;
        guint request_name_result;
        GError *error = NULL;

        proxy = dbus_g_proxy_new_for_name (self->priv->connection,
                                           DBUS_SERVICE_DBUS,
                                           DBUS_PATH_DBUS,
                                           DBUS_INTERFACE_DBUS);

        if (!org_freedesktop_DBus_request_name (proxy,
                                                MINFO_PROVIDER_SERVICE_NAME,
                                                0, &request_name_result,
                                                &error)) {
                g_warning ("Unable to register service: %s", error->message);
                g_error_free (error);
        }

        dbus_g_connection_register_g_object (self->priv->connection,
                                             MINFO_PROVIDER_SERVICE_OBJECT_PATH,
                                             G_OBJECT (self));

        g_object_unref (proxy);
}

static void
mvs_minfo_provider_service_set_property (GObject *object, guint property_id,
                                  const GValue *value, GParamSpec *pspec)
{
        MvsMInfoProviderService *self = MVS_MINFO_PROVIDER_SERVICE (object);

        switch (property_id) {
        case PROP_DBUSGCONN:
                if (!self->priv->connection) {
                        DBusGConnection *tmp = g_value_get_pointer (value);
                        if (tmp) {
                                self->priv->connection =
                                        dbus_g_connection_ref (tmp);
                                setup_dbus (self);
                        }
                }
                g_assert (self->priv->connection);
                break;
        default:
                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
        }
}

static void
mvs_minfo_provider_service_finalize (GObject *object)
{
        MvsMInfoProviderService *self = MVS_MINFO_PROVIDER_SERVICE (object);

        if (self->priv->connection) {
                dbus_g_connection_unref (self->priv->connection);
        }
        g_object_unref (self->priv->minfo_provider);
        G_OBJECT_CLASS (mvs_minfo_provider_service_parent_class)->finalize (object);
}

static void
mvs_minfo_provider_service_class_init (MvsMInfoProviderServiceClass *klass)
{
        GObjectClass *object_class = G_OBJECT_CLASS (klass);

        g_type_class_add_private (klass, sizeof (MvsMInfoProviderServicePrivate));

        object_class->set_property = mvs_minfo_provider_service_set_property;
        object_class->finalize = mvs_minfo_provider_service_finalize;

        g_object_class_install_property
                (object_class, PROP_DBUSGCONN,
                 g_param_spec_pointer ("connection", "DBusGConnection",
                                       "DBus GConnection",
                                       G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));

         dbus_g_object_type_install_info (MVS_TYPE_MINFO_PROVIDER_SERVICE,
                                          &dbus_glib_mvs_minfo_provider_service_object_info);

         mvs_minfo_provider_service_signals[RESPONSE_RECEIVED] =
                         g_signal_new ("response-received",
                         G_TYPE_FROM_CLASS (klass),
                         G_SIGNAL_RUN_LAST,
                         0,
                         NULL,
                         NULL,
                         g_cclosure_marshal_VOID__POINTER,
                         G_TYPE_NONE,
                         1,
                         G_TYPE_STRV,
                         NULL);
}

static void
mvs_minfo_provider_service_init (MvsMInfoProviderService *self)
{
        self->priv = GET_PRIVATE (self);
        self->priv->minfo_provider = mvs_minfo_provider_new ();
        self->priv->connection = NULL;
        self->priv->search_id = 0;

        g_signal_connect (self->priv->minfo_provider, "response-received",
                          G_CALLBACK (response_received_cb), self);
}

MvsMInfoProviderService*
mvs_minfo_provider_service_new (DBusGConnection *connection)
{
        return g_object_new (MVS_TYPE_MINFO_PROVIDER_SERVICE,
                             "connection", connection, NULL);
}
